/*
 * status.h --
 *
 *	Defines the statuses returned by Swarm functions. A status is
 *	defined in the following way:
 *
 *	MSB				LSB
 *	+-------+-------+-------+-------+
 *	|     Module	|    Message	|
 *	+-------+-------+-------+-------+
 *		16 bits		16 bits
 *
 *	Each module has its own set of statuses.
 *
 *	This scheme is derived from that used in the Sprite 
 *	operating system.
 *
 * Copyright 1999 Arizona Board of Regents
 *
 * $Header: /cvs/cvs/ftn/mirage/common/status.h,v 1.1 2001/12/12 23:03:31 bakers Exp $
 */

#ifndef _STATUS
#define _STATUS

/* 
 * This really should be int32, but including swarm.h leads to
 * circular dependencies.
 */
typedef unsigned int Status;

#define	STATUS_MODULE(status)		((status) >> 16)
#define STATUS_MSGNUM(status)		((status) & 0xFFFF)

/*
 * REMEMBER TO UPDATE STATUS.C
 */

/*
 * Status codes with module 0 are errnos. This makes it
 * backwards compatible.
 */

/*
 * Useful abbreviations. FAILURE will correspond to EPERM,
 * which may be confusing, so don't use FAILURE unless you're
 * lazy.
 */

#define SUCCESS				0x00000000
#define FAILURE				(!(SUCCESS))

/*
 * Swarm. The Swarm module is kind of a catch-all, so these are
 * kind of generic Swarm errors.
 */

#define SWARM_BAD_DEV_TYPE		0x00010000
#define SWARM_DUP_SID			0x00010001
#define SWARM_RESOLVE			0x00010002
#define SWARM_NO_DEVICE			0x00010003
#define SWARM_SERVICE_NAME_TOO_LONG	0x00010004
#define SWARM_INCOMPATIBLE_INTERFACE	0x00010005
#define SWARM_NOT_IMPLEMENTED		0x00010006
#define SWARM_UNAVAILABLE		0x00010007
#define SWARM_INVALID_FID		0x00010008
#define SWARM_INVALID_ARG		0x00010009
#define SWARM_IO_ERROR			0x0001000A
#define SWARM_INVALID_LEVEL		0x0001000B

/*
 * Storage devices
 */

#define DEV_NO_DEVICE			0x00020000
#define DEV_INVALID_ADDR		0x00020001


/* 
 * Status module itself
 */

#define STATUS_BADSTATUS		0x00030000

/*
 * Log module
 */

#define LOG_FRAG_ATTR_CORRUPTED		0x00040000
#define LOG_FRAG_ATTR_INVALID		0x00040001
#define LOG_RECORD_BAD_MAGIC		0x00040002
#define LOG_SECTION_BAD_MAGIC		0x00040003
#define LOG_SECTION_NOT_FOUND		0x00040004
#define LOG_SET_FULL			0x00040005
#define LOG_SET_DUPLICATE		0x00040006
#define LOG_SET_FROZEN			0x00040007
#define LOG_SET_DUPLICATE_RECORD	0x00040008
#define LOG_SET_RECORD_NOT_FOUND	0x00040009
#define LOG_RECORD_COMMITTED		0x0004000A
#define LOG_SERVICE_NOT_FOUND		0x0004000B
#define LOG_DUPLICATE_SERVICE		0x0004000C
/*
 * Striper
 */

#define STRIPER_NO_SID			0x00050000
#define STRIPER_ATTR_BAD_MAGIC		0x00050001
#define STRIPER_IO_ERROR		0x00050002
#define STRIPER_NO_STRIPES		0x00050003
#define STRIPER_NO_FID			0x00050004

/*
 * Storage
 */
#define STORAGE_NO_MARK			0x00060000
#define STORAGE_FID_EXISTS		0x00060001
#define STORAGE_NO_SPACE		0x00060002
#define STORAGE_NO_FID			0x00060003

/*
 * Agent
 */
#define AGENT_SKIP_REMAINING            0x00070000


extern char 	*Status_Msg(Status status);
extern Status	Status_ToErrno(Status status, int *errnoValue);

#endif /* _STATUS */

