/*
 * dos.c --
 *
 *      Old-style denial of service support for "invalid request attacks". No
 *      longer used.
 *
 * @COPYRIGHT@
 */

#include "nfsroute.h"

/*
 * GetRejectKeyFields: given an NFS error code, return a flag which describes
 * which fields should be used in the rejection key
 *
 */

int GetRejectKeyFields(int proc, int error, int *resetBy)
{
    switch (error) {
        case NFSERR_PERM:
            *resetBy = INVAL_SETATTR;
            return FIELDS_ProcVinAuth;

        case NFSERR_NOENT:
            *resetBy = INVAL_CREATEDIRENTS;
            return FIELDS_ProcVinFilename;

        case NFSERR_ACCES:
            *resetBy = INVAL_SETATTR;
            return FIELDS_ProcVinAuth;

        case NFSERR_EXIST:
            *resetBy = INVAL_REMOVEDIRENTS;
            if ((proc == NFSPROC_LINK) || (proc == NFSPROC_RENAME)) {
                return FIELDS_ProcVin2Filename2;
            } else {
                return FIELDS_ProcVinFilename;
            }

        case NFSERR_NOTDIR:
            if (proc == NFSPROC_RMDIR) {
                // special case rmdir since we don't know if the problem is
                // because the parent is not a dir, or the child is not a dir
                *resetBy = INVAL_CREATEDIRENTS;
                return FIELDS_ProcVinFilename;
            } else {
                *resetBy = 0;
                return FIELDS_ProcVin;
            }

        case NFSERR_ISDIR:
            if (proc == NFSPROC_REMOVE) {
                *resetBy = INVAL_CREATEDIRENTS;
                return FIELDS_ProcVinFilename;
            } else if (proc == NFSPROC_RENAME) {
                *resetBy = INVAL_REMOVEDIRENTS;
                return FIELDS_ProcVin2Filename2;
            } else {
                *resetBy = 0;
                return FIELDS_ProcVin;
            }

        case NFSERR_FBIG:
            *resetBy = INVAL_SETATTR;
            return FIELDS_ProcVin;

        case NFSERR_NOSPC:
            *resetBy = INVAL_LOWERQUOTA;
            return FIELDS_ProcVin;

        case NFSERR_ROFS:
            *resetBy = 0;
            return FIELDS_ProcVin;

        case NFSERR_NAMETOOLONG:
            *resetBy = 0;
            return FIELDS_ProcVinFilename;

        case NFSERR_NOTEMPTY:
            *resetBy = INVAL_REMOVEDIRENTSCHILD;
            return FIELDS_ProcVinFilename;

        case NFSERR_DQUOT:
            *resetBy = INVAL_LOWERQUOTA;
            return FIELDS_ProcVinAuth;

        default:
            return FIELDS_NONE;
    }
}

/*
 * DosCreateRejectKey
 *
 * Given the rejection fields (vin, proc, etc) and a nfs error code, create
 * the appropriate rejection key.
 *
 * Returns: SUCCESS if rejection key create
 */

Status DosCreateRejectKey(
        int vin,
        int proc,
        int uid,
        int gid,
        char *filename,
        int vin2,
        char *filename2,
        int error,
        TRejectKey *rejectKey,
        int *resetBy)
{
    int fields = GetRejectKeyFields(proc,error,resetBy);

    if (fields == FIELDS_NONE) {
        return FAILURE;
    }

    switch (fields) {
        case FIELDS_ProcVin:
            filename = REJECT_FILENAME_ANY;
            uid = REJECT_UID_ANY;
            gid = REJECT_GID_ANY;
            break;

        case FIELDS_ProcVinAuth:
            filename = REJECT_FILENAME_ANY;
            break;

        case FIELDS_ProcVinFilename:
            uid = REJECT_UID_ANY;
            gid = REJECT_GID_ANY;
            break;

        case FIELDS_ProcVin2Filename2:
            uid = REJECT_UID_ANY;
            gid = REJECT_GID_ANY;
            vin = vin2;
            filename = filename2;
            break;
    }

    return RejectCreateKey(vin,
        proc,
        uid,
        gid,
        filename,
        rejectKey);
}

/*
 * DosDoReject
 *
 * given a rejection key & rejection entry, construct the appropriate reply
 * and send it to the client. (Return TRUE)
 *
 * If the rejection has expired, delete it (Return FALSE)
 *
 */

int DosDoReject(
    TRejectKey *rejectKey,
    TRejectEntry *rejectEntry,
    char *rpcHeader,
    struct sockaddr_in *clientAddr,
    SendPacketFunc *sendFunc,
    void *sendFuncClientData)
{
    time_t timeTmp;
    char replyBuffer[MAX_REJECT_REPLY_SIZE];
    int replySize;
    char *auth, *nfsHeaderAddr;

    MirageDebug("  found rejection %s\n", RejectKeyToString(rejectKey));

    if (rejectEntry->expireTime <= MIRAGE_TIMESTAMP(&timeTmp)) {
        MirageDebug("  rejection expired %d/%d\n",
                rejectEntry->expireTime,
                MIRAGE_TIMESTAMP(&timeTmp));
        RejectRemove(rejectKey);
        return FALSE;
    }

    /* create the rejection reply message */

/*    replySize = rejectEntry->replySize;
    assert(replySize < sizeof(replyBuffer));
    memcpy(replyBuffer, rejectEntry->replyHeader, replySize);*/

    *RPC_XIDADDR(replyBuffer) = *RPC_XIDADDR(rpcHeader);
    *RPC_DIRECTION_ADDR(replyBuffer) = htonl(1);
    *RPC_REPLY_STATUS_ADDR(replyBuffer) = 0;

    // fill in an auth_none
    auth = RPC_REPLY_FIRSTAUTH_ADDR(replyBuffer);
    *OPAQUEAUTH_FLAVOR_ADDR(auth) = AUTH_NONE;
    *OPAQUEAUTH_SIZE_ADDR(auth) = 0;

    nfsHeaderAddr = OPAQUEAUTH_SKIP(auth,ntohl(*OPAQUEAUTH_SIZE_ADDR(auth)))+4;
    *((int*) (nfsHeaderAddr-4)) = 0; // XXX figure out what this is!

    if (DosExtendErrorCodes) {
        *NFS2_REPLY_STATUS_ADDR(nfsHeaderAddr) = htonl(rejectEntry->errorCode+100);
    } else {
        *NFS2_REPLY_STATUS_ADDR(nfsHeaderAddr) = htonl(rejectEntry->errorCode);
    }

    replySize = (nfsHeaderAddr-replyBuffer)+4;

    MirageDebug("  sending reject-reply to %s:%d\n",
            my_inet_ntoa(clientAddr->sin_addr.s_addr),
            clientAddr->sin_port);

    sendFunc(sendFuncClientData, replyBuffer, replySize, clientAddr,
         NULL, PATH_CLIENT_NFS);

    return TRUE;
}

/*
 * DosRequest
 *
 * returns TRUE if DOS reply was generated & sent to client
 *
 */

int DosRequest(char *rpcHeader,
    struct sockaddr_in *clientAddr,
    int uid,
    int gid,
    char *filename,
    int vin2,
    char *filename2,
    SendPacketFunc *sendFunc,
    void *sendFuncClientData)
{
    char *nfsHeader;
    char *firstAuth;
    int firstAuthSize;
    char *secondAuth;
    int secondAuthSize;
    int procNum;
    int vin;
    TRejectKey rejectKey;
    TRejectEntry *rejectEntry;

    if (!DosEnable) {
        return FALSE;
    }

    firstAuth = RPC_FIRSTAUTH_ADDR(rpcHeader);
    firstAuthSize = ntohl(*OPAQUEAUTH_SIZE_ADDR(firstAuth));
    secondAuth = OPAQUEAUTH_SKIP(firstAuth, firstAuthSize);
    secondAuthSize = ntohl(*OPAQUEAUTH_SIZE_ADDR(secondAuth));
    nfsHeader = OPAQUEAUTH_SKIP(secondAuth, secondAuthSize);

    procNum = ntohl(*RPC_PROCADDR(rpcHeader));
    vin = *NFS2_REQUEST_VIN_ADDR(nfsHeader);

    xdr_getuidgid(firstAuth, &uid, &gid);

    // try (proc, vin) rejections
    assert(RejectCreateKey(vin,
            procNum,
            REJECT_UID_ANY,
            REJECT_GID_ANY,
            REJECT_FILENAME_ANY,
            &rejectKey) == SUCCESS);

    if (RejectLookup(&rejectKey, &rejectEntry) == SUCCESS) {
        if (DosDoReject(&rejectKey, rejectEntry, rpcHeader, clientAddr,
                sendFunc, sendFuncClientData)) {
            return TRUE;
        }
    }

    // try (proc, vin, auth) rejections
    assert(RejectCreateKey(vin,
            procNum,
            uid,
            gid,
            REJECT_FILENAME_ANY,
            &rejectKey) == SUCCESS);

    if (RejectLookup(&rejectKey, &rejectEntry) == SUCCESS) {
        if (DosDoReject(&rejectKey, rejectEntry, rpcHeader, clientAddr,
                sendFunc, sendFuncClientData)) {
            return TRUE;
        }
    }

    if (filename != NULL) {
        // try (proc, vin, filename) rejections
        assert(RejectCreateKey(vin,
                procNum,
                REJECT_UID_ANY,
                REJECT_GID_ANY,
                filename,
                &rejectKey) == SUCCESS);

        if (RejectLookup(&rejectKey, &rejectEntry) == SUCCESS) {
            if (DosDoReject(&rejectKey, rejectEntry, rpcHeader, clientAddr,
                    sendFunc, sendFuncClientData)) {
                return TRUE;
            }
        }
    }

    if (filename2 != NULL) {
        // try (proc, vin2, filename2) rejections
        assert(RejectCreateKey(vin2,
                procNum,
                REJECT_UID_ANY,
                REJECT_GID_ANY,
                filename2,
                &rejectKey) == SUCCESS);

        if (RejectLookup(&rejectKey, &rejectEntry) == SUCCESS) {
            if (DosDoReject(&rejectKey, rejectEntry, rpcHeader, clientAddr,
                    sendFunc, sendFuncClientData)) {
                return TRUE;
            }
        }
    }

    return FALSE;
}

void DosInvalidate(TTransactEntry *transactEntry, char *nfsHeaderPtr)
{
    int proc = transactEntry->proc;
    int vin = transactEntry->vin;
    int pvin = transactEntry->pvin;
    int uid = transactEntry->uid;
    int gid = transactEntry->gid;
    char *filename = transactEntry->filename;
    int vin2 = transactEntry->vin2;
    char *filename2 = transactEntry->filename2;

    if ((proc == NFSPROC_MKDIR) || (proc == NFSPROC_CREATE)) {
//        RejectInvalidateVin(*NFS2_DIROPRES_VIN_ADDR(nfsHeaderPtr));
    }

    switch (proc) {
        case NFSPROC_SETATTR:
            RejectInvalidate(INVAL_SETATTR, proc, vin, REJECT_UID_ANY,
                    REJECT_GID_ANY, REJECT_FILENAME_ANY);
            break;

        case NFSPROC_MKDIR:
            MirageDebug("proc=%d vin=%d filename=%s", proc, vin, filename);
        case NFSPROC_CREATE:
        case NFSPROC_SYMLINK:
            RejectInvalidate(INVAL_CREATEDIRENTS, proc, vin, REJECT_UID_ANY,
                    REJECT_GID_ANY, filename);
            break;

        case NFSPROC_LINK:
            MirageDebug("vin=%d fn = %s\n", vin2, filename2);
            RejectInvalidate(INVAL_CREATEDIRENTS, proc, vin2, REJECT_UID_ANY,
                    REJECT_GID_ANY, filename2);
            break;

        case NFSPROC_REMOVE:
        case NFSPROC_RMDIR:
            MirageDebug("pvin = %d/%d uid=%d gid=%d\n",pvin,ntohl(pvin),uid,gid);
            RejectInvalidate(INVAL_REMOVEDIRENTSCHILD, proc, pvin, REJECT_UID_ANY,
                    REJECT_GID_ANY, REJECT_FILENAME_ANY);
            RejectInvalidate(INVAL_REMOVEDIRENTS, proc, vin, REJECT_UID_ANY,
                    REJECT_GID_ANY, filename);
            RejectInvalidate(INVAL_LOWERQUOTA, proc, REJECT_VIN_ANY, uid, gid,
                    REJECT_FILENAME_ANY);
            break;

        case NFSPROC_RENAME:
            MirageDebug("proc=%d vin=%d fn=%s vin2=%d fn2=%s\n", proc, vin, filename, vin2, filename2);
            RejectInvalidate(INVAL_REMOVEDIRENTS, proc, vin, REJECT_UID_ANY,
                    REJECT_GID_ANY, filename);
            RejectInvalidate(INVAL_CREATEDIRENTS, proc, vin2, REJECT_UID_ANY,
                    REJECT_GID_ANY, filename2);
            break;
    }
}

void DosReply(TTransactEntry *transactPtr,
    char *rpcHeader,
    int rpcHeaderLen,
    char *nfsHeader,
    struct sockaddr_in *clientAddr)
{
    TRejectKey rejectKey;
    int status = ntohl(*NFS2_REPLY_STATUS_ADDR(nfsHeader));
    time_t timeTmp;
    int vin = transactPtr->vin;
    int procNum = transactPtr->proc;
    int invalType;

    if (!DosEnable) {
        return;
    }

    if (procNum == NFSPROC_NULL) {
        return;
    }

    if (status == NFS_OK) {
        DosInvalidate(transactPtr, nfsHeader);
        return;
    }

    if (DosCreateRejectKey(vin,
            procNum,
            transactPtr->uid,
            transactPtr->gid,
            transactPtr->filename,
            transactPtr->vin2,
            transactPtr->filename2,
            status,
            &rejectKey,
            &invalType) != SUCCESS) {
        return;
    }

    assert(RejectCreate(&rejectKey,
            status,
            MIRAGE_TIMESTAMP(&timeTmp)+30,
            rpcHeader,
            rpcHeaderLen,
            invalType) == SUCCESS);

    MirageDebug("  create rejection %s\n",
        RejectKeyToString(&rejectKey));
}






