/*
 *  The Mirage NFS Router
 *
 *  mirageTime: various helper functions for time
 *
 *  @COPYRIGHT@
 *
 */

#include "nfsroute.h"

/*
 * TimeGetS
 *
 * Get the current time, in seconds.
 *
 */

int TimeGetS()
{
    struct timeval tv;
    struct timezone tz;

#ifdef MIRAGE_SCOUT
    do_gettimeofday(&tv);
#else
    gettimeofday(&tv, &tz);
#endif

    return tv.tv_sec;
}

/*
 * TimeGetMS
 *
 * Get the current time, in milliseconds. Time is returned as an int
 * and may overflow.
 *
 */

int TimeGetMS()
{
    struct timeval tv;
    struct timezone tz;

#ifdef MIRAGE_SCOUT
    do_gettimeofday(&tv);
#else
    gettimeofday(&tv, &tz);
#endif

    return tv.tv_sec*10000+tv.tv_usec/100;
}

/*
 * TimeGetMSLong
 *
 * Get the current time, in milliseconds. Returned as a long-long to prevent
 * overflow
 *
 */

unsigned long long TimeGetMSLong()
{
    struct timeval tv;
    struct timezone tz;

#ifdef MIRAGE_SCOUT
    do_gettimeofday(&tv);
#else
    gettimeofday(&tv, &tz);
#endif

    return (((unsigned long long) tv.tv_sec) * 1000) +
            (((unsigned long long) tv.tv_usec)/1000);
}

/*
 * TimeGetUS
 *
 * Get the current time, in microseconds. Returned as a long-long to prevent
 * overflow.
 *
 */

unsigned long long TimeGetUS()
{
    struct timeval tv;
    struct timezone tz;

#ifdef MIRAGE_SCOUT
    do_gettimeofday(&tv);
#else
    gettimeofday(&tv, &tz);
#endif

    return (((unsigned long long) tv.tv_sec) * 1000000) +
            (((unsigned long long) tv.tv_usec)/1);
}

/*
 * GetTimeStamp
 *
 * return the pentium cycle counter. might overflow.
 *
 */

int GetTimeStamp()
{
  unsigned long long time;

  time=0;

  asm("rdtsc":"=&A"(time));

  return (time & 0xFFFFFFFF);
}
