/*
 *  The Mirage NFS Router
 *
 *  @COPYRIGHT@
 *
 *   nfs.h: Definitions for the NFS and MOUNT protocols.
 *
 */

#ifndef nfsH
#define nfsH

#define NFS2_FHSIZE     32

#define NFSPROC_NULL            0
#define NFSPROC_GETATTR         1
#define NFSPROC_SETATTR         2
#define NFSPROC_ROOT            3
#define NFSPROC_LOOKUP          4
#define NFSPROC_READLINK        5
#define NFSPROC_READ            6
#define NFSPROC_WRITECACHE      7
#define NFSPROC_WRITE           8
#define NFSPROC_CREATE          9
#define NFSPROC_REMOVE          10
#define NFSPROC_RENAME          11
#define NFSPROC_LINK            12
#define NFSPROC_SYMLINK         13
#define NFSPROC_MKDIR           14
#define NFSPROC_RMDIR           15
#define NFSPROC_READDIR         16
#define NFSPROC_STATFS          17
#define MIRAGEPROC_IOCTL        100

#define NFS_OK                  0
#define NFSERR_PERM             1
#define NFSERR_NOENT            2
#define NFSERR_IO               5
#define NFSERR_NXIO             6
#define NFSERR_ACCES            13
#define NFSERR_EXIST            17
#define NFSERR_NODEV            19
#define NFSERR_NOTDIR           20
#define NFSERR_ISDIR            21
#define NFSERR_FBIG             27
#define NFSERR_NOSPC            28
#define NFSERR_ROFS             30
#define NFSERR_NAMETOOLONG      63
#define NFSERR_NOTEMPTY         66
#define NFSERR_DQUOT            69
#define NFSERR_STALE            70
#define NFSERR_WFLUSH           99

#define NFS_MNT_PROGRAM         100005
#define NFS_MNT_VERSION         1
#define MNTPROC_NULL            0
#define MNTPROC_MNT             1
#define MNTPROC_UMNT            3
#define MNTPROC_UMNTALL         4
#define MNTPROC_EXPORTS         5

#define NFS3_FHSIZE             64

#define NFS_MAXFHSIZE           64

#define MIRAGE_IOCTL_DOS_EXTEND_ERROR 1
#define MIRAGE_IOCTL_DOS_RESET 2

struct nfs_fh {
        unsigned short          size;
        unsigned char           data[NFS_MAXFHSIZE];
};

struct mnt_fhstatus {
        unsigned int            status;
        struct nfs_fh *         fh;
};

/* macros to get addresses of rpc fields */
#define RPC_XIDADDR(x) ((int*) (x))
#define RPC_DIRECTION_ADDR(x) ((int*) ((x)+4))
#define RPC_PROCADDR(x) ((int*) ((x)+20))

#define RPC_REPLY_STATUS_ADDR(x) ((int*) ((x)+8))
/* for accepted replies */
#define RPC_REPLY_FIRSTAUTH_ADDR(x) ((x)+12)
/* for rejected replies */
#define RPC_REPLY_REJSTATUS_ADDR(x) ((int*) ((x)+12))
#define RPC_REPLY_REJAUTHSTATUS_ADDR(x) ((int*) ((x)+16))

#define NFS2_REQUEST_VIN_ADDR(x) ((int*) (x))
#define NFS2_REQUEST_MOUNTCHECKSUM_ADDR(x) ((int*) ((x)+16))
#define NFS2_REQUEST_DOSID_ADDR(x) ((int*) ((x)+20))
#define NFS2_REQUEST_HVC_ADDR(x) ((int*) ((x)+24))
//#define NFS2_REQUEST_PVIN_ADDR(x) ((int*) ((x)+24))

#define NFS2_DIROPARGS_FILENAME_LEN_ADDR(x) ((int*) ((x)+32))
#define NFS2_DIROPARGS_FILENAME_ADDR(x) ((x) + 36)

#define NFS2_LINKARGS_VIN2_ADDR(x) ((int*) ((x)+NFS2_FHSIZE))
#define NFS2_LINKARGS_FILENAME_LEN_ADDR(x) ((int*) ((x)+NFS2_FHSIZE+NFS2_FHSIZE))
#define NFS2_LINKARGS_FILENAME_ADDR(x) ((x)+NFS2_FHSIZE+NFS2_FHSIZE+4)

#define NFS2_RENAMEARGS_TO_ADDR(x) (NFS2_DIROPARGS_FILENAME_ADDR(x) + XDR_QUADLEN(ntohl(*NFS2_DIROPARGS_FILENAME_LEN_ADDR(x)))*4)
#define NFS2_RENAMEARGS_VIN2_ADDR(x) ((int*) NFS2_RENAMEARGS_TO_ADDR(x))
#define NFS2_RENAMEARGS_FILENAME2_LEN_ADDR(x) (NFS2_DIROPARGS_FILENAME_LEN_ADDR(NFS2_RENAMEARGS_TO_ADDR(x)))
#define NFS2_RENAMEARGS_FILENAME2_ADDR(x) (NFS2_DIROPARARGS_FILENAME_ADDR(NFS2_RENAMEARGS_TO_ADDR(x)))

#define NFS2_REPLY_STATUS_ADDR(x) ((int*) (x))

#define NFS2_DIROPOK_FSID_ADDR(x) ((int*) ((x) + 32 + 36))
#define NFS2_DIROPOK_FILEID_ADDR(x) ((int*) ((x) + 32 + 40))

#define NFS2_ATTRSTAT_FSID_ADDR(x) ((int*) ((x) + 36))
#define NFS2_ATTRSTAT_FILEID_ADDR(x) ((int*) ((x) + 40))

#define NFS2_DIROPRES_VIN_ADDR(x) ((int*) ((x) + 4))

#define NFS2_IOCTL_WHICH_ADDR(x) ((int*) (x))
#define NFS2_IOCTL_VALUE_ADDR(x) ((int*) ((x)+4))

#define MNT_MNTPATHLEN_ADDR(x) ((int*) (x))
#define MNT_MNTPATHDATA_ADDR(x) (x+4)

#define RPC_FIRSTAUTH_ADDR(x) ((x) + 6*4)
#define OPAQUEAUTH_FLAVOR_ADDR(x) ((int*) (x))
#define OPAQUEAUTH_SIZE_ADDR(x) ((int*) (x+4))
#define OPAQUEAUTH_DATA_ADDR(x) ((x+2*4))
#define OPAQUEAUTH_SKIP(x, size) (x + 2*4 + ((size==0) ? 0 : (XDR_QUADLEN(size)*4)))

//---------------------------------------------------------------------------
#endif










