#ifndef _NFS_RPCGEN_TYPES_H_RPCGEN
#define _NFS_RPCGEN_TYPES_H_RPCGEN

#include <rpc/types.h>

#ifdef MIRAGE_SCOUT
#include <xdr/xdr.h>
#else
#include <rpc/xdr.h>
#endif

#define MNTPATHLEN 1024
#define MNTNAMLEN 255
#define FHSIZE 32
#define FHSIZE3 64

#define MIRAGE_IOCTL_DOS_EXTEND_ERROR 1
#define MIRAGE_IOCTL_DOS_RESET 2
#define MIRAGE_IOCTL_DOS_ENABLE 3
#define MIRAGE_IOCTL_DISABLE_QUEUEDELAY 4
#define MIRAGE_IOCTL_DISABLE_VALIDATE 5
#define MIRAGE_IOCTL_DISABLE_SCHEDULER 6
#define MIRAGE_IOCTL_DUMP_STATS 7
#define MIRAGE_IOCTL_SET_NFS_PORT 8
#define MIRAGE_IOCTL_DISABLE_RECOVERY 9

typedef char fhandle[FHSIZE];

typedef struct {
	u_int fhandle3_len;
	char *fhandle3_val;
} fhandle3;

typedef char *dirpath;

typedef char *name;

struct fhstatus {
	u_int status;
	union {
		fhandle directory;
	} fhstatus_u;
};
typedef struct fhstatus fhstatus;

enum mountstat3 {
	MNT3_OK = 0,
	MNT3ERR_PERM = 1,
	MNT3ERR_NOENT = 2,
	MNT3ERR_IO = 5,
	MNT3ERR_ACCES = 13,
	MNT3ERR_NOTDIR = 20,
	MNT3ERR_INVAL = 22,
	MNT3ERR_NAMETOOLONG = 63,
	MNT3ERR_NOTSUPP = 10004,
	MNT3ERR_SERVERFAULT = 10006,
};
typedef enum mountstat3 mountstat3;

struct mountres3_ok {
	fhandle3 fhandle;
	struct {
		u_int auth_flavors_len;
		int *auth_flavors_val;
	} auth_flavors;
};
typedef struct mountres3_ok mountres3_ok;

struct mountres3 {
	mountstat3 fhs_status;
	union {
		mountres3_ok mountinfo;
	} mountres3_u;
};
typedef struct mountres3 mountres3;

typedef struct mountbody *mountlist;

struct mountbody {
	name ml_hostname;
	dirpath ml_directory;
	mountlist ml_next;
};
typedef struct mountbody mountbody;

typedef struct groupnode *groups;

struct groupnode {
	name gr_name;
	groups gr_next;
};
typedef struct groupnode groupnode;

typedef struct exportnode *exports;

struct exportnode {
	dirpath ex_dir;
	groups ex_groups;
	exports ex_next;
};
typedef struct exportnode exportnode;

#ifdef NFS_OK
typedef int nfsstat;
#else
enum nfsstat {
	NFS_OK = 0,
	NFSERR_PERM = 1,
	NFSERR_NOENT = 2,
	NFSERR_IO = 5,
	NFSERR_NXIO = 6,
	NFSERR_ACCES = 13,
	NFSERR_EXIST = 17,
	NFSERR_NODEV = 19,
	NFSERR_NOTDIR = 20,
	NFSERR_ISDIR = 21,
	NFSERR_FBIG = 28,
	NFSERR_NOSPC = 28,
	NFSERR_NOFS = 30,
	NFSERR_NAMETOOLONG = 63,
	NFSERR_NOTEMPTY = 66,
	NFSERR_DQUOT = 69,
	NFSERR_STALE = 70,
	NFSERR_WFLUSH = 99,
};
typedef enum nfsstat nfsstat;
#endif

enum ftype {
	NFNON = 0,
	NFREG = 1,
	NFDIR = 2,
	NFBLK = 3,
	NFCHR = 4,
	NFLNK = 5,
};
typedef enum ftype ftype;

struct nfstimeval {
	u_int seconds;
	u_int useconds;
};
typedef struct nfstimeval nfstimeval;

struct fattr {
	ftype type;
	u_int mode;
	u_int nlink;
	u_int uid;
	u_int gid;
	u_int size;
	u_int blocksize;
	u_int rdev;
	u_int blocks;
	u_int fsid;
	u_int fileid;
	nfstimeval atime;
	nfstimeval mtime;
	nfstimeval ctime;
};
typedef struct fattr fattr;

struct sattr {
	u_int mode;
	u_int uid;
	u_int gid;
	u_int size;
	nfstimeval atime;
	nfstimeval mtime;
};
typedef struct sattr sattr;

typedef char *filename;

typedef char *path;

typedef struct {
	u_int nfsdata_len;
	char *nfsdata_val;
} nfsdata;

typedef char nfscookie[4];

struct attrstat {
	nfsstat status;
	union {
		fattr attributes;
	} attrstat_u;
};
typedef struct attrstat attrstat;

struct diropargs {
	fhandle dir;
	filename name;
};
typedef struct diropargs diropargs;

struct sdiropok {
	fhandle file;
	fattr attributes;
};
typedef struct sdiropok sdiropok;

struct diropres {
	nfsstat status;
	union {
		sdiropok diropok;
	} diropres_u;
};
typedef struct diropres diropres;

struct sattrargs {
	fhandle file;
	sattr attributes;
};
typedef struct sattrargs sattrargs;

struct readlinkres {
	nfsstat status;
	union {
		path data;
	} readlinkres_u;
};
typedef struct readlinkres readlinkres;

struct readargs {
	fhandle file;
	u_int offset;
	u_int count;
	u_int totalcount;
};
typedef struct readargs readargs;

struct readres_ok {
	fattr attributes;
	nfsdata data;
};
typedef struct readres_ok readres_ok;

struct readres {
	nfsstat status;
	union {
		readres_ok readok;
	} readres_u;
};
typedef struct readres readres;

struct writeargs {
	fhandle file;
	u_int beginoffset;
	u_int offset;
	u_int totalcount;
	nfsdata data;
};
typedef struct writeargs writeargs;

struct createargs {
	diropargs where;
	sattr attributes;
};
typedef struct createargs createargs;

struct renameargs {
	diropargs from;
	diropargs to;
};
typedef struct renameargs renameargs;

struct linkargs {
	fhandle from;
	diropargs to;
};
typedef struct linkargs linkargs;

struct symlinkargs {
	diropargs from;
	path to;
	sattr attributes;
};
typedef struct symlinkargs symlinkargs;

struct readdirargs {
	fhandle dir;
	nfscookie cookie;
	u_int count;
};
typedef struct readdirargs readdirargs;

typedef struct entry *pentry;

struct entry {
	u_int fileid;
	filename name;
	nfscookie cookie;
	pentry nextentry;
};
typedef struct entry entry;

struct sreaddirok {
	entry *entries;
	bool_t eof;
};
typedef struct sreaddirok sreaddirok;

struct readdirres {
	nfsstat status;
	union {
		sreaddirok readdirok;
	} readdirres_u;
};
typedef struct readdirres readdirres;

struct sinfo {
	u_int tsize;
	u_int bsize;
	u_int blocks;
	u_int bfree;
	u_int bavail;
};
typedef struct sinfo sinfo;

struct statfsres {
	nfsstat status;
	union {
		sinfo info;
	} statfsres_u;
};
typedef struct statfsres statfsres;

struct fhandleargs {
	fhandle data;
};
typedef struct fhandleargs fhandleargs;

struct ioctlargs {
	int which;
	int value;
};
typedef struct ioctlargs ioctlargs;

extern  bool_t xdr_fhandle (XDR *, fhandle);
extern  bool_t xdr_fhandle3 (XDR *, fhandle3*);
extern  bool_t xdr_dirpath (XDR *, dirpath*);
extern  bool_t xdr_name (XDR *, name*);
extern  bool_t xdr_fhstatus (XDR *, fhstatus*);
extern  bool_t xdr_mountstat3 (XDR *, mountstat3*);
extern  bool_t xdr_mountres3_ok (XDR *, mountres3_ok*);
extern  bool_t xdr_mountres3 (XDR *, mountres3*);
extern  bool_t xdr_mountlist (XDR *, mountlist*);
extern  bool_t xdr_mountbody (XDR *, mountbody*);
extern  bool_t xdr_groups (XDR *, groups*);
extern  bool_t xdr_groupnode (XDR *, groupnode*);
extern  bool_t xdr_exports (XDR *, exports*);
extern  bool_t xdr_exportnode (XDR *, exportnode*);
extern  bool_t xdr_nfsstat (XDR *, nfsstat*);
extern  bool_t xdr_ftype (XDR *, ftype*);
extern  bool_t xdr_nfstimeval (XDR *, nfstimeval*);
extern  bool_t xdr_fattr (XDR *, fattr*);
extern  bool_t xdr_sattr (XDR *, sattr*);
extern  bool_t xdr_filename (XDR *, filename*);
extern  bool_t xdr_path (XDR *, path*);
extern  bool_t xdr_nfsdata (XDR *, nfsdata*);
extern  bool_t xdr_nfscookie (XDR *, nfscookie);
extern  bool_t xdr_attrstat (XDR *, attrstat*);
extern  bool_t xdr_diropargs (XDR *, diropargs*);
extern  bool_t xdr_sdiropok (XDR *, sdiropok*);
extern  bool_t xdr_diropres (XDR *, diropres*);
extern  bool_t xdr_sattrargs (XDR *, sattrargs*);
extern  bool_t xdr_readlinkres (XDR *, readlinkres*);
extern  bool_t xdr_readargs (XDR *, readargs*);
extern  bool_t xdr_readres_ok (XDR *, readres_ok*);
extern  bool_t xdr_readres (XDR *, readres*);
extern  bool_t xdr_writeargs (XDR *, writeargs*);
extern  bool_t xdr_createargs (XDR *, createargs*);
extern  bool_t xdr_renameargs (XDR *, renameargs*);
extern  bool_t xdr_linkargs (XDR *, linkargs*);
extern  bool_t xdr_symlinkargs (XDR *, symlinkargs*);
extern  bool_t xdr_readdirargs (XDR *, readdirargs*);
extern  bool_t xdr_pentry (XDR *, pentry*);
extern  bool_t xdr_entry (XDR *, entry*);
extern  bool_t xdr_sreaddirok (XDR *, sreaddirok*);
extern  bool_t xdr_readdirres (XDR *, readdirres*);
extern  bool_t xdr_sinfo (XDR *, sinfo*);
extern  bool_t xdr_statfsres (XDR *, statfsres*);
extern  bool_t xdr_fhandleargs (XDR *, fhandleargs*);
extern  bool_t xdr_ioctlargs (XDR *, ioctlargs*);

#endif /* !_NFS_RPCGEN_H_RPCGEN */
