//---------------------------------------------------------------------------

#ifndef nfspartnerH
#define nfspartnerH

#include <status.h>
#include "nfs.h"
#include "nfs_rpcgen_types.h"

#define PATH_CLIENT_NFS 0
#define PATH_CLIENT_MOUNT 1
#define PATH_SERVER_NFS 2
#define PATH_SERVER_MOUNT 3

typedef Status (SendPacketFunc) (
    void *clientData,
    char *packetPtr,
    int packetLen,
    struct sockaddr_in *clientAddr,
    struct TServer *server,
    int whichPath);

typedef Status (PortMapFunc) (
    void *clientData,
    struct sockaddr_in *serverAddr,
    int prog,
    int vers,
    int *port);

typedef Status (RecoverySetupFunc) (
    void *clientData,
    struct sockaddr_in mountAddr,
    struct sockaddr_in nfsAddr);

typedef Status (RecoveryExportFunc) (
    void *clientData,
    exports *exportsResult);

typedef Status (RecoveryGetAttrFunc) (
    void *clientData,
    char *fh,
    attrstat *attrStat);

typedef Status (RecoveryMountFunc) (
    void *clientData,
    char *mountPath,
    fhstatus *mountResult);

typedef Status (RecoveryLookupFunc) (void *clientData,
    char *dirHandle,
    char *fn,
    diropres *lookupResult);

typedef Status (RecoveryReadDirFunc) (void *clientData,
    char *dirHandle,
    char *cookie,
    readdirres *readdirRes);

typedef Status (RecoveryCleanupFunc) (void *clientData);

typedef struct TNfsPartner {
    PortMapFunc *portMapFunc;
    SendPacketFunc *sendPacketFunc;

    RecoverySetupFunc *recoverySetupFunc;
    RecoveryExportFunc *exportFunc;
    RecoveryMountFunc *mountFunc;
    RecoveryLookupFunc *lookupFunc;
    RecoveryGetAttrFunc *getAttrFunc;
    RecoveryReadDirFunc *readDirFunc;
    RecoveryCleanupFunc *recoveryCleanupFunc;
} TNfsPartner;

//---------------------------------------------------------------------------
#endif










