//---------------------------------------------------------------------------

#ifndef nfsrouteH
#define nfsrouteH

#include <hash.h>
#include <list.h>
#include <status.h>
#include "nfs.h"

#define MAX_SERVERS 2

#ifdef MIRAGE_SCOUT
#include <sys/kernel_iface.h>
//#define MirageDebug(message...)
#define MirageDebug(message...) traceLog(MIRAGE, TR_DETAILED, ##message);
#define MirageError(message...) traceLog(MIRAGE, TR_ERRORS, ##message);
#define MiragePrint(message...) traceLog(MIRAGE, TR_ERRORS, ##message);
#define MIRAGE_TIMESTAMP(x) 0
#else
#define MIRAGE_QUIET 0
#define MIRAGE_ERROR 1
#define MIRAGE_DEBUG 2
#define MirageDebug(message...) if (MirageLogLevel>=MIRAGE_DEBUG) fprintf(stderr, ##message)
#define MirageError(message...) if (MirageLogLevel>=MIRAGE_ERROR) fprintf(stderr, ##message)
#define MiragePrint(message...) fprintf(stderr, ##message)
#define MIRAGE_TIMESTAMP(x) time(x)
extern int MirageLogLevel;
#endif


#ifdef MIRAGE_SCOUT
#define AF_INET 2
struct sockaddr_in {
    int sin_family;
    struct { int s_addr; } sin_addr;
    int sin_port;
};
#endif

/*
 * TServer structure. This holds all information (Addresses, ports, etc)
 *    for one server.
 *
 */

typedef struct TServer {
    int sid;
    char *address;
    int mountPort;
    int nfsPort;
    struct sockaddr_in nfsAddr;
    struct sockaddr_in mountAddr;
    struct Path *mountPath;
    struct Path *nfsPath;
} TServer;

/* Servers are stored in an array. */

extern TServer Servers[MAX_SERVERS];
extern int ServerCount;

extern int SecureOnly;
extern int DisableValidate;
extern int DisableQueueDelay;
extern int DisableScheduler;
extern int DisableRecovery;
extern int DosExtendErrorCodes;
extern int DosEnable;

/* Socket variables for the mount protocol */

extern int mountSocket;
extern int mountOutSocket;
extern struct sockaddr_in mountAddr;
extern struct sockaddr_in mountOutAddr;

/* Socket variables for the nfs protocol */

extern int nfsSocket;
extern int nfsOutSocket;
extern struct sockaddr_in nfsAddr;
extern struct sockaddr_in nfsOutAddr;

#include "mirageTime.h"
#include "nfspartner.h"
#include "umt.h"
#include "tt.h"
#include "ht.h"
#include "rt.h"
#include "sht.h"
#include "xdr.h"
#include "dos.h"
#include "recovery.h"
#include "ioctl.h"
#include "score.h"
#include "schedule.h"

#define NUM_NFS_REQNAMES 18
extern char *NfsReqNames[NUM_NFS_REQNAMES];

// the following variables are basically a backdoor to let the Silk side of
// mirage have access to some state
extern int lastProcNum;
extern TTransactEntry *lastTransactEntry;
extern TServer *lastServer;

char *my_inet_ntoa(int inaddr);

TServer *AddServer(char *address,
    int mountPort,
    int nfsPort,
    PortMapFunc *portMapFunc,
    void *clientData);

void HandleMountRequest(
    TNfsPartner *partner,
    void *clientData,
    char *rpcHeaderPtr,
    int packetLen,
    struct sockaddr_in *fromAddr);

void HandleMountReply(
    TNfsPartner *partner,
    void *clientData,
    char *rpcHeaderPtr,
    int packetLen,
    struct sockaddr_in *fromAddr);

void HandleNfsRequest(
    TNfsPartner *partner,
    void *clientData,
    void *dosClientData,
    char *rpcHeaderPtr,
    int packetLen,
    struct sockaddr_in *fromAddr);

Status HandleNfsReply(
    TNfsPartner *partner,
    void *clientData,
    char *rpcHeaderPtr,
    int packetLen,
    struct sockaddr_in *fromAddr);

Status ValidateRpcRequest(char *packetPtr,
    int packetLen,
    int *dosId,
    int *dosScore);

//---------------------------------------------------------------------------
#endif










