/*
 *  The Mirage NFS Router
 *
 *  @COPYRIGHT@
 *
 *  nfstable.c: implements the mapping table between VH and PH
 *
 * XXX OBSOLETE XXX
 *
 * this is from a really old version of mirage
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <rpc/rpc.h>
#include "nfstable.h"

TableNode *FirstTableNode = NULL;
int VFileIdCounter = 0;

/*
 * TableInitVH
 *
 *     Given a TableNode, create its virutal handle.
 */

void TableInitVH(TableNode *tNode)
{
    int *p = (int*) &tNode->vh.data;

    memset(p, 0, NFS_MAXFHSIZE);
    *(p++) = tNode->vfsid;
    *(p++) = tNode->vfileId;
    *(p++) = tNode->server;

    tNode->vh.size = NFS2_FHSIZE;
}

/*
 * TableLookup
 *
 *     Look up a TableNode by various search criteria. 'flags' contains one
 *     or more FLAG_* bits indicating which fields will be searched.
 *
 * Results:
 *     The TableNode matching the search criteria, or NULL if none exists.
 */

TableNode *TableLookup(
    int flags,
    int fileId,
    int fsid,
    int server,
    struct nfs_fh *ph,
    struct nfs_fh *vh)
{
    TableNode *tNode;

    for (tNode = FirstTableNode; tNode != NULL; tNode = tNode->next) {
        if ((flags & FLAG_FILEID) && (fileId != tNode->fileId)) {
            continue;
        }

        if ((flags & FLAG_FSID) && (fsid != tNode->fsid)) {
            continue;
        }

        if ((flags & FLAG_SERVER) && (server != tNode->server)) {
            continue;
        }

        if ((flags & FLAG_PH) &&
                ((ph->size != tNode->ph.size) ||
                        (memcmp(ph->data, tNode->ph.data, ph->size)!=0))) {
            continue;
        }

        if ((flags & FLAG_VH) &&
                ((vh->size != tNode->vh.size) ||
                        (memcmp(vh->data, tNode->vh.data, vh->size)!=0))) {
            continue;
        }

        return tNode;
    }

    return NULL;
}

/*
 * TableUpdatePH
 *
 *     Search for a TableNode, based on the fields specified by 'flags'. Create
 *     the TableNode if it doesn't exist. Update any of the fields that are
 *     contained in 'flags'.
 *
 * Results:
 *     The TableNode that matches the search criteria (created if necessary)
 *
 * Side Effects:
 *     TableNode fields updated; New node added to list if necessary.
 */


TableNode *TableUpdatePH(
    int flags,
    int fileId,
    int fsid,
    int server,
    struct nfs_fh *ph)
{
    TableNode *tNode = TableLookup(flags, fileId, fsid, server, ph, NULL);

    if (tNode == NULL) {
        tNode = (TableNode*) malloc(sizeof(TableNode));

        tNode->flags = 0;
        tNode->fileId = 0;
        tNode->fsid = 0;
        tNode->server = server;

        tNode->vfsid = 0;
        tNode->vfileId = VFileIdCounter++;
        TableInitVH(tNode);

        tNode->next = FirstTableNode;
        FirstTableNode = tNode;
    }

    if (flags & FLAG_FILEID) {
        tNode->flags |= FLAG_FILEID;
        tNode->fileId = fileId;
    }

    if (flags & FLAG_FSID) {
        tNode->flags |= FLAG_FSID;
        tNode->fsid = fsid;
    }

    if (flags & FLAG_PH) {
        tNode->flags |= FLAG_PH;
        tNode->ph = *ph;
    }

    return tNode;
}

/*
 * TableGetVH
 *
 *      Really just a wrapper around TableUpdatePH.
 */

int TableGetVH(
    int flags,
    int fileId,
    int fsid,
    int server,
    struct nfs_fh *ph,
    int *vfsid,
    int *vfileid,
    struct nfs_fh *vh)
{
    TableNode *tNode;

    tNode = TableUpdatePH(flags | FLAG_SERVER, fileId, fsid, server, ph);

    if (tNode == NULL) {
        return FALSE;
    }

    if (vfsid) {
        *vfsid = tNode->vfsid;
    }

    if (vfileid) {
        *vfileid = tNode->vfileId;
    }

    if (vh) {
        *vh = tNode->vh;
    }

    return TRUE;
}

/*
 * TableGetServerFromVH
 *
 *     Given a VH, return the server number. XXX this could be done by
 *     looking at the server number that is embedded in the VH rather than
 *     searching the table.
 *
 * Results:
 *     TRUE if successful. *server will get the server number
 */

int TableGetServerFromVH(struct nfs_fh *vh,
    int *server)
{
    TableNode *tNode;

    tNode = TableLookup(FLAG_VH, 0, 0, 0, NULL, vh);

    if (tNode != NULL) {
        *server = tNode->server;
        return TRUE;
    } else {
        return FALSE;
    }
}

/*
 * TableGetPH
 *
 *     Given a VH, search the table to find the appropriate PH.
 *
 * Results:
 *     TRUE if successful. *ph will get the PH.
 */


int TableGetPH(struct nfs_fh *vh,
    struct nfs_fh *ph)
{
    TableNode *tNode;

    tNode = TableLookup(FLAG_VH, 0, 0, 0, NULL, vh);

    if (tNode == NULL) {
        return FALSE;
    }

    if (ph) {
        *ph = tNode->ph;
    }

    return TRUE;
}
