/*
 * rt.c --
 *
 *      Implements the rejection table.
 *
 *      OBSOLETE. Used by the "invalid request attack" dos mechanism
 *
 * @COPYRIGHT@
 */

#include "nfsroute.h"

Hash_Table RejectHashTable;

Status RejectInit()
{
    Hash_InitTable(&RejectHashTable, 0, sizeof(TRejectKey)/4);

    return SUCCESS;
}

Status RejectCreateKey(int vin, int procedure, int uid, int gid,
        char *filename, TRejectKey *key)
{
    memset(key, 0, sizeof(TRejectKey));

    key->vin = vin;
    key->procNum = procedure;
    key->uid = uid;
    key->gid = gid;

    if (filename) {
        if (strlen(filename) >= MAX_REJECT_FILENAME) {
            return FAILURE;
        }
        strcpy(key->filename, filename);
    }

    return SUCCESS;
}

Status RejectLookup(TRejectKey *rejectKey, TRejectEntry **rejectEntry)
{
    Hash_Entry *h;

    h = Hash_FindEntry(&RejectHashTable, (void*) rejectKey);
    if (h == NULL) {
        return FAILURE;
    }

    *rejectEntry = (TRejectEntry*) Hash_GetValue(h);
    if (*rejectEntry == NULL) {
        return FAILURE;
    }

    return SUCCESS;
}

Status RejectCreate(
    TRejectKey *key,
    int errorCode,
    int expireTime,
    char *replyHeader,
    int replySize,
    int invalType)
{
    Hash_Entry *h;
    Boolean isNew;
    TRejectEntry *entry;

    entry = (TRejectEntry*) MALLOC(sizeof(TRejectEntry));

    entry->errorCode = errorCode;
    entry->expireTime = expireTime;
    entry->invalType = invalType;

    assert(replySize < MAX_REJECT_REPLY_SIZE);
    memcpy(entry->replyHeader, replyHeader, replySize);
    entry->replySize = replySize;

    h = Hash_CreateEntry(&RejectHashTable, (void*) key, &isNew);
    Hash_SetValue(h, entry);

    return SUCCESS;
}

Status RejectRemove(TRejectKey *rejectKey)
{
    Hash_Entry *h;

    h = Hash_FindEntry(&RejectHashTable, (void*) rejectKey);
    if (h == NULL) {
        return FAILURE;
    }

    Hash_DeleteEntry(&RejectHashTable, h);

    return SUCCESS;
}

Status RejectInvalidate(
        int invalType,
        int proc,
        int vin,
        int uid,
        int gid,
        char *filename)
{
    Hash_Entry *h;
    Hash_Search hashSearchPtr;

    h = Hash_EnumFirst(&RejectHashTable,&hashSearchPtr);
    while (h!=NULL) {
        TRejectKey *rejectKey = (TRejectKey*) Hash_GetKey(&RejectHashTable,
                h);
        TRejectEntry *rejectEntry = (TRejectEntry*) Hash_GetValue(h);

        if (rejectEntry->invalType != invalType) {
            // do nothing -- failed to match reset mask
        } else if ((vin != REJECT_VIN_ANY) && (rejectKey->vin != vin)) {
            // do nothing -- failed to match vin
        } else if ((uid != REJECT_UID_ANY) && (rejectKey->uid != uid)) {
            // do nothing -- failed to match uid
        } else if ((gid != REJECT_GID_ANY) && (rejectKey->gid != gid)) {
            // do nothing -- faield to match gid
        } else if ((filename != REJECT_FILENAME_ANY) &&
                (strcmp(rejectKey->filename, filename)!=0)) {
            // do nothing -- failed ot match filename
        } else {
            // must have matched all criteria

            MirageDebug( "  invalidating reject entry %s\n",
                    RejectKeyToString(rejectKey));
            Hash_DeleteEntry(&RejectHashTable, h);
        }
        h = Hash_EnumNext(&hashSearchPtr);
    }

    return SUCCESS;
}

char *RejectKeyToString(TRejectKey *rejectKey)
{
    static char build[1024];
    char tmp[1024];

    sprintf(build, "(proc=%d, vin=%d", rejectKey->procNum, rejectKey->vin);

    if (rejectKey->uid != REJECT_UID_ANY) {
        sprintf(tmp, ", uid=%d", ntohl(rejectKey->uid));
        strcat(build, tmp);
    }

    if (rejectKey->gid != REJECT_GID_ANY) {
        sprintf(tmp, ", gid =%d", ntohl(rejectKey->gid));
        strcat(build, tmp);
    }

    if (*rejectKey->filename) {
        sprintf(tmp, ", fn=%s", rejectKey->filename);
        strcat(build, tmp);
    }

    strcat(build,")");

    return build;
}
