//---------------------------------------------------------------------------

#ifndef scheduleH
#define scheduleH

#include <status.h>

#define SCHED_MAX_REQUESTS_PER_CLIENT 16

#define SCHED_INITIAL_BALANCE 1000
#define SCHED_MAX_BALANCE 1000
#define SCHED_SHARES 500

#define STOLEN_HANDLE_PENALTY 100

#define MAX_QUEUE_SLOTS SCHED_MAX_REQUESTS_PER_CLIENT

typedef struct TSchedRequest TSchedRequest;

typedef void (DestroyRequestFunc) (TSchedRequest *req);

typedef struct TCircularQueue {
    void *slots[MAX_QUEUE_SLOTS];
    int size;
    int head;
    int tail;
    int count;
} TCircularQueue;

struct TSchedRequest {
    int clientId;
    int schedCost;
    char *packetPtr;
    int packetLen;
    struct sockaddr_in fromAddr;

    // silk stuff
    struct Iface *silkSelf;
    struct Msg *msg;
    struct mirage_connect_state *connectState;
    int whichPath;
    int isActive;

    DestroyRequestFunc *destroyFunc;
};

typedef struct TSchedHeader {
    int clientId;
    int bankBalance;
    int shares;
    TCircularQueue requestQueue;
    struct TSchedHeader *next;

    int head;
    int tail;
    int count;
} TSchedHeader;

void SchedFreeRequest(TSchedRequest *req);

TSchedRequest *SchedInitRequest(int dosId,
    int dosScore,
    DestroyRequestFunc *destroyFunc);

void SchedEnqueue(TSchedRequest *request);

Status SchedPopRequest(
    TSchedRequest **request);

extern int SchedNumWaiting;

//---------------------------------------------------------------------------
#endif










