/*
 *  The Mirage NFS Router
 *
 *  @COPYRIGHT@
 *
 *  sht.c: stolen handle table
 *
 */

#include "nfsroute.h"

#define SHT_FREELIST

int ShtXidCounter = 0;
Hash_Table ShtHashTable;

TShtEntry *ShtFreeList;
int LastShtXid;

/*
 * StolenHandleInit
 *
 * Initialize this module
 *
 */

Status StolenHandleInit()
{
    Hash_InitTable(&ShtHashTable, 0, HANDLE_HASH_SIZE/4);
    ShtFreeList = NULL;

    return SUCCESS;
}

/*
 * StolenHandleEntryAlloc
 *
 * Allocate an entry from the freelist.
 *
 */

TShtEntry *ShtEntryAlloc()
{
    TShtEntry *entry;

    if (ShtFreeList!=NULL) {
        entry = ShtFreeList;
        ShtFreeList = ShtFreeList->next;
    } else {
        entry = (TShtEntry*) MALLOC(sizeof(TShtEntry));
    }

    return entry;
}

/*
 * StolenHandleEntryFree
 *
 * Return an entry to the freelist.
 *
 */

void StolenHandleEntryFree(TShtEntry *entry)
{
#ifdef SHT_FREELIST
    entry->next = ShtFreeList;
    ShtFreeList = entry;
#else
    FREE(entry);
#endif
}

/*
 * StolenHandleAddToHash
 *
 * Add a SHT entry to the SHT hash table
 *
 */

Status StolenHandleAddToHash(char *hvcPtr, TShtEntry *shtEntry)
{
    Hash_Entry *h;
    Boolean isNew;

    h = Hash_CreateEntry(&ShtHashTable, (void*) hvcPtr, &isNew);
    Hash_SetValue(h, shtEntry);

    shtEntry->h = h;

    return SUCCESS;
}

/*
 * StolenHandleCreate
 *
 * Create a new SHT entry
 *
 */

Status StolenHandleCreate(char *hvcPtr, TShtEntry **shtEntryOut)
{
    TShtEntry *shtEntry;

    if (StolenHandleLookup(hvcPtr, &shtEntry)) {
        shtEntry->count++;
    } else {
        shtEntry = ShtEntryAlloc();
        shtEntry->count = 0;
        shtEntry->refCount = 1;
        shtEntry->h = NULL;

        StolenHandleAddToHash(hvcPtr, shtEntry);
    }

    if (shtEntryOut) {
        *shtEntryOut = shtEntry;
    }

    return SUCCESS;
}

/*
 * StolenHandleLookup
 *
 * Search the SHT for an Xid, and return the corresponding Sht entry
 *
 */

Status StolenHandleLookup(char *hvcPtr, TShtEntry **shtEntryOut)
{
    Hash_Entry *h;
    TShtEntry *shtEntry;

    h = Hash_FindEntry(&ShtHashTable, (void*) hvcPtr);
    if (h == NULL) {
        return FAILURE;
    }

    shtEntry = (TShtEntry*) Hash_GetValue(h);
    if (shtEntry == NULL) {
        return FAILURE;
    }

    if (shtEntryOut) {
        *shtEntryOut = shtEntry;
    }

    return SUCCESS;
}

Status StolenHandleFree(TShtEntry *shtEntry)
{
    shtEntry->refCount--;
    if (shtEntry->refCount <= 0) {
        Hash_Entry *h = shtEntry->h;
        StolenHandleEntryFree(shtEntry);

        if (h != NULL) {
            Hash_DeleteEntry(&ShtHashTable, h);
        }
    }
}



