/*
 *  The Mirage NFS Router
 *
 *  @COPYRIGHT@
 *
 *  ttSilk.c
 *
 *  NOTE: Any changes to this file should also be made to tt.c !!
 *
 * Implements a modified version of the TT for silk. Since silk has per-path
 * state, there's no reason for a TT hash table, so this file implements stubs
 * that put the TT entry into the per-path state.
 *
 */

#include "nfsroute.h"

extern TTransactEntry *SilkTransactEntry;
extern TTransactKey *SilkTransactKey;

Status TransactInit()
{
    return SUCCESS;
}

Status TransactCreateKey(int xid, int clientIp, TTransactKey *key)
{
    key->xid = xid;
    key->clientIp = clientIp;

    return SUCCESS;
}

Status TransactCreate(TTransactKey *key,
    int proc,
    int vin,
    int mountChecksum,
    int uid,
    int gid,
    int dosId,

    TTransactEntry **entryOut)
{
    TTransactEntry *entry = SilkTransactEntry;

    entry->proc = proc;
    entry->vin = vin;
    entry->mountChecksum = mountChecksum;
    entry->uid = uid;
    entry->gid = gid;
    entry->dosId = dosId;
    entry->refCount = 1;
    entry->h = NULL;

    // invalid dos fields
    entry->pvin = -1;
    *entry->filename = 0;
    entry->vin2 = -1;
    *entry->filename2 = 0;

    if (entryOut) {
        *entryOut = entry;
    }

    *SilkTransactKey = *key;

    return SUCCESS;
}

Status TransactFree(TTransactEntry *entry)
{
    return SUCCESS;
}

Status TransactLookup(TTransactKey *key, TTransactEntry **entry)
{
    if (memcmp(key, SilkTransactKey, sizeof(TTransactKey)) != 0) {
        return FAILURE;
    }

    (*entry) = SilkTransactEntry;

    return SUCCESS;
}


