/*
 *  The Mirage NFS Router
 *
 *  @COPYRIGHT@
 *
 *  xdr.c: functions to encode and decode XDR variables
 *
 */

#include "nfsroute.h"
//#include <assert.h>
//#include "xdr.h"

/*
 * XDR procedures. Based on information and code from the linux kernel sources.
 *
 * Each procedure returns a pointer to the byte following the field it has
 * decoded (i.e. to where the next field will be)
 *
 */

#define XDR_IN_L(x) ntohl(x)


char *xdr_decode_opaqueauth(char *inp)
{
    int *p = (int*) inp;
    int size;

    p++; // skip auth_flavor;
    size = ntohl(*p);

    return inp + 2*4 + ((size==0) ? 0 : (XDR_QUADLEN(size)*4));
}

char *xdr_decode_rpcrequest(char *inp, int *xid, int *proc) //struct rpc_msg *rpcHeader)
{
    int *p = (int*) inp;

    /*rpcHeader->rm_xid =*/ *xid = XDR_IN_L(*p++);
    /*rpcHeader->rm_direction =*/ XDR_IN_L(*p++);
    /*rpcHeader->ru.RM_cmb.cb_rpcvers =*/ XDR_IN_L(*p++);
    /*rpcHeader->ru.RM_cmb.cb_prog  =*/ XDR_IN_L(*p++);
    /*rpcHeader->ru.RM_cmb.cb_vers =*/ XDR_IN_L(*p++);
    /*rpcHeader->ru.RM_cmb.cb_proc =*/ *proc =  XDR_IN_L(*p++);

    inp = xdr_decode_opaqueauth(inp + 6*4);
    inp = xdr_decode_opaqueauth(inp);

    return inp;
}

char *xdr_decode_rpcreply(char *inp, int *xid, int *stat)
{
    int *p = (int*) inp;

    /*rpcHeader->rm_xid = */ *xid = XDR_IN_L(*p++);
    /*rpcHeader->rm_direction =*/ XDR_IN_L(*p++);

    /*rpcHeader->ru.RM_rmb.rp_stat =*/ *stat =  XDR_IN_L(*p++);

    if ( *stat==0) {
        p = (int*) xdr_decode_opaqueauth((char*) p);
        /*rpcHeader->ru.RM_rmb.ru.RP_ar.ar_stat =*/ XDR_IN_L(*p++);
    } else {
        /*rpcHeader->ru.RM_rmb.ru.RP_dr.rj_stat =*/ XDR_IN_L(*(p++));
        /*rpcHeader->ru.RM_rmb.ru.RP_dr.ru.RJ_versions.low =*/ XDR_IN_L(*(p++));
        /*rpcHeader->ru.RM_rmb.ru.RP_dr.ru.RJ_versions.high =*/ XDR_IN_L(*(p++));
    }


    return (char*) p;
}

char *xdr_decode_nfsstat(char *inp, int *nfsstat)
{
    *nfsstat = XDR_IN_L(*((int*) inp));

    return inp+4;
}

char *xdr_decode_fhandle(char *inp, struct nfs_fh *fh)
{
        memset((void *)fh, 0, sizeof(*fh));

        fh->size = NFS2_FHSIZE;
        memcpy(fh->data, inp, NFS2_FHSIZE);

        return inp + NFS2_FHSIZE;
}

char *xdr_decode_fhstatus(char *inp, struct mnt_fhstatus *res)
{
        int *p = (int*) inp;
        struct nfs_fh *fh = res->fh;

        memset((void *)fh, 0, sizeof(*fh));
        if ((res->status = ntohl(*p++)) == 0) {
            fh->size = NFS2_FHSIZE;
            memcpy(fh->data, p, NFS2_FHSIZE);
            return inp + 4 + fh->size;
        } else {
            return inp + 4;
        }
}

char *xdr_decode_fhstatus3(char *inp, struct mnt_fhstatus *res)
{
    int *p = (int*) inp;
    struct nfs_fh *fh = res->fh;

    memset((void *)fh, 0, sizeof(*fh));
    if ((res->status = ntohl(*p++)) == 0) {
        int size = ntohl(*p++);
        if (size <= NFS3_FHSIZE) {
            fh->size = size;
            memcpy(fh->data, p, size);
            return inp + 4 + 4 + size;
        } else {
            res->status = -1;
            return inp + 4 + 4;
        }
    } else {
        return inp + 4;
    }
} 

char *xdr_encode_handle(char *outp, struct nfs_fh *fh)
{
    assert(fh->size <= NFS2_FHSIZE);

    memcpy(outp, fh->data, NFS2_FHSIZE);

    return outp + NFS2_FHSIZE;
}

char *xdr_encode_handle3(char *outp, struct nfs_fh *fh)
{
    int *p = (int*) outp;

    assert(fh->size <= NFS3_FHSIZE);

    *(p++) = htonl(fh->size);
    memcpy(p, fh->data, fh->size);

    return outp + 4 + fh->size;
}

int xdr_getuidgid(char *inp, int *uid, int *gid)
{
    int *p = (int*) inp;
    int flavor = ntohl(*(p++));
    int size = ntohl(*(p++));
    int machineNameLen;

    *uid = -1;
    *gid = -1;

    if (flavor != AUTH_UNIX) {
        return FALSE;
    }

    if (size < 16) {
        return FALSE;
    }

    p++; // skip timestamp
    machineNameLen = ntohl(*(p++));

    p+=XDR_QUADLEN(machineNameLen);

    if (inp-((char*) p)+8 > size) {
        return FALSE;
    }

    *uid = *(p++);
    *gid = *(p++);

    return TRUE;
}






